#ifndef __SheppLoganFilter_h
#define __SheppLoganFilter_h 
#include <vector> 


//ITK includes
#include "itkImageToImageFilter.h"
#include "itkImageRegionIterator.h"
#include "itkConstNeighborhoodIterator.h"
#include "itkOffset.h" 

/*! 
 * \class SheppLoganFilter 
 * \brief Applies a shepp logan filter to an image and returns the filtered image. 
 */

template < typename TInputImage,  typename TOutputImage >
class SheppLoganFilter : public itk::ImageToImageFilter< TInputImage, TOutputImage >
{
public:    
/*! Standard class typedefs. */    
typedef SheppLoganFilter Self;    
typedef itk::ImageToImageFilter< TInputImage, TOutputImage > Superclass;    
typedef itk::SmartPointer< Self > Pointer;    
typedef itk::SmartPointer< const Self > ConstPointer;     
/*! Method for creation through object factory */    
itkNewMacro(Self);     
/*! Run-time type information (and related methods) */    
itkTypeMacro(SheppLoganFilter, ImageToImageFilter);     
//Convenience typedefs    
typedef TInputImage ImageType;
typedef TOutputImage OImageType;    
typedef typename ImageType::RegionType RegionType;
typedef itk::ConstNeighborhoodIterator< ImageType > NeighborhoodIteratorType;
typedef itk::ImageRegionIterator< OImageType> IteratorType;
typedef typename ImageType::ConstPointer ImageConstPointer;    
typedef typename OImageType::Pointer ImagePointer;     
/*! Extract dimension from input and output image. */    
itkStaticConstMacro(ImageDimension, unsigned int, TInputImage::ImageDimension);     
itkSetMacro(VRadius, int);    
itkGetConstReferenceMacro(VRadius, int); 

protected:    
SheppLoganFilter();    
virtual ~SheppLoganFilter() {}    
/*! Standard "PrintSelf" method */    
void PrintSelf(std::ostream& os, itk::Indent indent) const;     
/*!     
* SheppLoganFilter can be implemented as a      
* multithreaded filter. Therefore, this implementation provides     
* a ThreadedGenerateData() routine which is called for each      
* processing thread. The output image data is allocated      
* automatically by the superclass prior to calling      
* ThreadedGenerateData().  ThreadedGenerateData can only     
* write to the portion of the output image specified by the     
* parameter "outputRegionForThread".     
*     
* \sa ImageToImageFilter::ThreadedGenerateData(),     
*     ImageToImageFilter::GenerateData()     
*/    
void ThreadedGenerateData(const RegionType& regionForThread,                               
 int threadId ); 

private:    
SheppLoganFilter(const Self&) {}; //purposely not implemented    
void operator=(const Self&); //purposely not implemented     
/*! vertical radius of shepp logan filter  */    
int m_VRadius;
}; 


#ifndef ITK_MANUAL_INSTANTIATION
#include "SheppLoganFilter.txx"
#endif 

#endif 
