#ifndef _SheppLoganFilter_txx
#define _SheppLoganFilter_txx 

#include "SheppLoganFilter.h" 

#ifndef PI

#define PI 3.141592653

#endif 

template < typename TInputImage, typename TOutputImage >
SheppLoganFilter< TInputImage, TOutputImage >::SheppLoganFilter()
{
m_VRadius = 0;
} 

template < typename TInputImage, typename TOutputImage >
void SheppLoganFilter< TInputImage, TOutputImage >
::ThreadedGenerateData(const RegionType&  
  regionForThread, int threadId )
{
if(m_VRadius <= 0)
  {
  itkExceptionMacro(<< "Member variable VRadius not properly initialized.");
  }

//pointers to output and input image
ImageConstPointer inputImage  = this->GetInput();
ImagePointer outputImage = this->GetOutput();
RegionType inputRegion = inputImage->GetLargestPossibleRegion();
RegionType outputRegion = outputImage->GetLargestPossibleRegion();
IteratorType out(outputImage, outputRegion);

typename NeighborhoodIteratorType::RadiusType radius;

radius.Fill(0);
radius[1] = m_VRadius;
int vDim = m_VRadius*2+1;
NeighborhoodIteratorType it(radius, inputImage, inputRegion);
//pointer to Shepp-Logan filter weights
float *sl = new float[vDim];
//vector with the OffsetTypes for the neighborhood iterator

typedef typename NeighborhoodIteratorType::OffsetType OffsetType;

std::vector< OffsetType > offset;

int actIndex;
for(int i = -m_VRadius; i <= m_VRadius; ++i)
{
actIndex = i + m_VRadius;
*(sl+actIndex) = -2.0/(PI*PI*(4*i*i-1));
OffsetType tmpO = {{0,i}};
offset.push_back(tmpO);
}
std::vector< int > tmpImg;
for (it.GoToBegin(), out.GoToBegin(); !it.IsAtEnd(); ++it, ++out)
{
float sum = 0;
for(int i = 0; i < m_VRadius*2+1; ++i)
{
sum += *(sl+i) * it.GetPixel(offset[i]);
}
tmpImg.push_back(sum);
}
int i = 0;
for (it.GoToBegin(), out.GoToBegin(); !it.IsAtEnd(); ++it, ++out)
{
out.Set(tmpImg[i++]);
}
} 


template < typename TInputImage, typename TOutputImage>

void SheppLoganFilter< TInputImage, TOutputImage >::PrintSelf(std::ostream& os, itk::Indent indent) const
{
Superclass::PrintSelf(os,indent);
os << "VRadius = " << m_VRadius << std::endl;
} 


#endif 
 
